<?xml version="1.0" encoding="UTF-8"?>
<!--
/*
   CRT - Classic - shader
   
   Copyright (C) 2018 guest(r) - guest.r@gmail.com

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

-->
<shader language="GLSL">

<vertex><![CDATA[
    uniform vec2 rubyTextureSize;

    void main()
    {
            gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;
            gl_TexCoord[0] = gl_MultiTexCoord0;
    }
]]></vertex>

	
<fragment outscale = "1.0"  filter="linear"><![CDATA[
	
uniform sampler2D rubyTexture;
uniform vec2 rubyInputSize;
uniform vec2 rubyOutputSize;
uniform vec2 rubyTextureSize;

vec2 dxx = vec2(1.0/rubyTextureSize.x, 0.0);
vec2 dy  = vec2(0.0, 1.0/rubyTextureSize.y);

#define mask        0.0     // 0.0 - Aperture, 1.0 - scattered mask
#define glow        0.04
#define brightboost 1.10
#define gammaOUT    0.40
#define h_sharp     4.50    // pixel sharpness
#define scanline    9.00    // scanline param, vertical sharpness
#define WarpX       0.04    // x-curvature setting, 0.0 - 0.1
#define WarpY       0.03    // y-curvature setting, 0.0 - 0.1
#define CORNER
	
#define beam_min   1.6	  // dark area beam min - wide
#define beam_max   2.3	  // bright area beam max - narrow


float sw(float x, float l)
{
	float d = x;
	float bm = scanline;
	float b = mix(beam_min,beam_max,l);
	d = exp2(-bm*pow(d,b));
	return d;
}

float l(float x)
{
	return exp2(-h_sharp*x*x);
}

vec3 Mask(float pos){

	float m = fract(pos/3.0);
	if (m < 1.0/3.0) return vec3(1.0, 0.0, 0.0); else
	if (m < 2.0/3.0) return vec3(0.0, 1.0, 0.0); else	
	return vec3(0.0, 0.0, 1.0);
}    

// Lottes curvature

vec2 Warp(vec2 pos){
  pos=pos*2.0-1.0;    
  pos*=vec2(1.0+(pos.y*pos.y)*WarpX,1.0+(pos.x*pos.x)*WarpY);
  return pos*0.5+0.5;}

/*  CRT shader - corner function

    Copyright (C) 2010-2012 cgwg, Themaister and DOLLS

    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the Free
    Software Foundation; either version 2 of the License, or (at your option)
    any later version.

    (cgwg gave their consent to have the original version of this shader
    distributed under the GPL in this message:

        http://board.byuu.org/viewtopic.php?p=26075#p26075

        "Feel free to distribute my shaders under the GPL. After all, the
        barrel distortion code was taken from the Curvature shader, which is
        under the GPL."
    )
*/
	
float corner(vec2 coord)
{
                coord *= rubyTextureSize / rubyInputSize;
                coord = (coord - vec2(0.5)) * 1.0 + vec2(0.5);
                coord = min(coord, vec2(1.0)-coord) * vec2(1.0, rubyInputSize.y/rubyInputSize.x);
                vec2 cdist = vec2(0.025);
                coord = (cdist - min(coord,cdist));
                float dist = sqrt(dot(coord,coord));
                return clamp((cdist.x-dist)*350.0,0.0, 1.0);
}  
  
void main()
{
	// Calculating texel coordinates
	
	vec2 texcoord = Warp(gl_TexCoord[0].xy*(rubyTextureSize/rubyInputSize))*(rubyInputSize/rubyTextureSize); 
	
	vec2 inv_size = 1.0/rubyTextureSize;
	vec2 OGL2Pos = texcoord * rubyTextureSize;
	vec2 fp = fract(OGL2Pos);
	vec2 dx = vec2(inv_size.x,0.0);
	vec2 dy = vec2(0.0,inv_size.y);
	
	vec2 pC4 = floor(OGL2Pos) * inv_size + 0.5*inv_size;	

	vec3 tl1 = texture2D(rubyTexture,pC4 -     dx - dy).xyz; tl1*=tl1;
	vec3 tct = texture2D(rubyTexture,pC4          - dy).xyz; tct*=tct;
	vec3 tr1 = texture2D(rubyTexture,pC4 +     dx - dy).xyz; tr1*=tr1;
	
	vec3 ul2 = texture2D(rubyTexture,pC4 - dx -dx).xyz; ul2*=ul2;
	vec3 ul1 = texture2D(rubyTexture,pC4 -     dx).xyz; ul1*=ul1;
	vec3 uct = texture2D(rubyTexture,pC4         ).xyz; uct*=uct;
	vec3 ur1 = texture2D(rubyTexture,pC4 +     dx).xyz; ur1*=ur1;
	vec3 ur2 = texture2D(rubyTexture,pC4 + dx +dx).xyz; ur2*=ur2;
	
	vec3 bl1 = texture2D(rubyTexture,pC4 -     dx + dy).xyz; bl1*=bl1;
	vec3 bct = texture2D(rubyTexture,pC4          + dy).xyz; bct*=bct;
	vec3 br1 = texture2D(rubyTexture,pC4 +     dx + dy).xyz; br1*=br1;
	
	float wl2 = l(fp.x+1.5);
	float wl1 = l(fp.x+0.5);
	float wct = l(fp.x-0.5);	
	float wr1 = l(1.5-fp.x);
	float wr2 = l(2.5-fp.x);
	
	vec3 color0 = (wl1*tl1 + wct*tct + wr1*tr1)/(wl1+wct+wr1);	
	vec3 color1 = (wl2*ul2 + wl1*ul1 + wct*uct + wr1*ur1 + wr2*ur2)/(wl2+wl1+wct+wr1+wr2);	
	vec3 color2 = (wl1*bl1 + wct*bct + wr1*br1)/(wl1+wct+wr1);	
	
// calculating scanlines
	
	float sw2 = sw(1.5-fp.y, max(max(color2.r,color2.g),color2.b));
	float sw1 = sw(0.5-fp.y, max(max(color1.r,color1.g),color1.b));
	float sw0 = sw(fp.y+0.5, max(max(color0.r,color0.g),color0.b));
	
	vec3 color = color2*sw2 + color1*sw1 + color0*sw0;	
	vec3 color3 = color/(sw2+sw1+sw0);
	
// applying mask

	float msk = 1.2;	
	
	color3 = pow(color3, vec3(msk));	
	color3 = color3 * Mask(gl_FragCoord.x + mask*mod(gl_FragCoord.y,2.0));

#ifdef CORNER
	color3 = pow(color3, vec3(gammaOUT))*corner(texcoord);	
#else
	color3 = pow(color3, vec3(gammaOUT));	
#endif
	
	gl_FragColor = vec4(color3*brightboost,sw0+sw1+sw2);	

	}
    ]]></fragment>
	
<vertex><![CDATA[

void main(void) {
    gl_Position = ftransform();
    gl_TexCoord[0] = gl_MultiTexCoord0;
}
]]></vertex>

// Bloom pass

<fragment scale="1.0" filter="nearest"><![CDATA[
uniform sampler2D rubyTexture;
uniform vec2 rubyInputSize;
uniform vec2 rubyOutputSize;
uniform vec2 rubyTextureSize;

#define brightboost 1.0		

const float sqrt3     = 1.732050807568877;

vec3 gamma_correct(vec3 color, float gamma)
{
	float l = length(color)/sqrt3;
	l = pow(l,gamma)*sqrt3;
	return l*normalize(color);
}

float intensity (vec3 A) 
{
	return clamp((1.0 - pow(max(max(A.r,A.g),A.b),0.5))*4.5, 0.325, 3.0);
}

float l(float x, float i)
{
	return exp2(-i*x);
}
  
void main()
{	
	vec4 O = texture2D(rubyTexture, gl_TexCoord[0].xy);
	
	vec2 inv_size = 1.0/rubyTextureSize;	
	vec2 dx = vec2(inv_size.x,0.0);
	
	vec3 ul2 = texture2D(rubyTexture,gl_TexCoord[0].xy - dx -dx).xyz; ul2*=ul2;
	vec3 ul1 = texture2D(rubyTexture,gl_TexCoord[0].xy -     dx).xyz; ul1*=ul1;
	vec3 uct = texture2D(rubyTexture,gl_TexCoord[0].xy         ).xyz; uct*=uct;
	vec3 ur1 = texture2D(rubyTexture,gl_TexCoord[0].xy +     dx).xyz; ur1*=ur1;
	vec3 ur2 = texture2D(rubyTexture,gl_TexCoord[0].xy + dx +dx).xyz; ur2*=ur2;

	float wl2 = l(4.0, intensity(ul2));
	float wl1 = l(1.0, intensity(ul1));
	float wct = 1.0;	
	float wr1 = l(1.0, intensity(ur1));
	float wr2 = l(4.0, intensity(ur2));	

	vec3 E = (wl2*ul2 + wl1*ul1 + wct*uct + wr1*ur1 + wr2*ur2);		
	
	E = min(E,1.0);
	E = E*O.a;
	E = sqrt(E);
	
	//E = gamma_correct(E, 0.85);	
	E = pow(E, vec3(0.85));
	
	gl_FragColor = vec4(E*brightboost,1.0);		
}	
]]></fragment>


</shader>
